/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.builder;

import java.util.Hashtable;
import java.util.NoSuchElementException;
import java.util.Stack;
import java.util.StringTokenizer;
import javax.swing.SwingWorker;
import tp.model.DocRepresentTable;
import tp.model.DocumentsDatabase;
import tp.model.DocsRepresentDB;
import tp.model.Document;

/**
 * Class, which makes representation of text data by N-grams.
 * @author Ondrej Sabatka
 */

public class BuilderNGram extends SwingWorker<Void, Void> implements BuilderInterface {

    //Clenske promenne tridy
    private DocsRepresentDB.RepresentationModel model;
    private DocsRepresentDB.RepresentationFeature feature;
    private DocsRepresentDB.Preprocessing preprocessing;

    private int min_ngram_occur;


    /**
     * Constructor of the class
     * @param model model used for representation (binary, TF or TF/IDF)
     * @param preprocessing preprocessing options selected by the user
     */
    public BuilderNGram(DocsRepresentDB.RepresentationModel model, DocsRepresentDB.Preprocessing preprocessing) {
        this.model = model;
        this.preprocessing = preprocessing;
        this.feature = DocsRepresentDB.RepresentationFeature.NGRAMS;

        this.min_ngram_occur = 1;
    }


    //************************************************************************//
    //                          CLASSES FOR SUFFIX TREE                         //
    //************************************************************************//
    /**
     * Root of the suffix tree
     */
    private class RootNode {
        Hashtable<String, ChildNode> child_nodes;

        //Constructor
        public RootNode() {
            this.child_nodes = new Hashtable<String, ChildNode>();
        }
    }


    /**
     * Nodes of the suffix tree
     */
    private class ChildNode {
        String word;
        String ngram;
        int count_occur;
        int depth;
        Hashtable<String, ChildNode> child_nodes;
        
        static final int FIRST_OCCUR = 1;
        static final int ROOT_DEPTH = 1;

        //constructor
        public ChildNode(String word, int count_occur, int depth) {
            this.word = word;
            this.ngram = word;
            this.count_occur = count_occur;
            this.depth = depth;
            this.child_nodes = new Hashtable<String, ChildNode>();
        }
    }

    /**
     * N-grams found in the tree
     */
    private class NGrams {
        String ngram;
        int occur;

        //constructor
        public NGrams(String ngram, int occur) {
            this.ngram = ngram;
            this.occur = occur;
        }
    }



    /**
     * Gets text documents from a database and creates their N-gram representation.
     * @param database input database of text documents
     * @param depth the lentgh of N-grams
     * @return database of all document representations
     */

    public DocsRepresentDB buildRepresentation(DocumentsDatabase database, int depth) {
        DocsRepresentDB representation = new DocsRepresentDB(model, feature, preprocessing);

        //Sets settings about depth and minimum occurence of N-grams
        representation.setNGramDepth(depth);
        representation.setMinNGramOccur(min_ngram_occur);

        //Takes all documents and creates their representation
        for(int i = 0; i < database.getCountDocuments(); i++) {
            Document text_doc = database.getDocumentAt(i);

            //Document is cut into N-grams
            Stack<NGrams> ngrams;
            ngrams = cutDocumentToNGrams(text_doc.getContentDocument(), depth);
            

             //Table for document representations is created
            DocRepresentTable doc_table = new DocRepresentTable();

            //Individual N-grams are inserted into the table of representations
            while(!ngrams.empty()) {
                NGrams ngram = ngrams.pop();

                if(this.model == DocsRepresentDB.RepresentationModel.BINARY) {
                    doc_table.addFeature(ngram.ngram);
                }
                else {
                    doc_table.addFeature(ngram.ngram, ngram.occur);
                }
            }

            //Document class is stored into the table
            doc_table.setTopic(text_doc.getTopic());

            //Insertion of table with document representation into database of all representations
            representation.addDocument(doc_table);

            //Setting of a progress bar
            setProgress(i * 100 / database.getCountDocuments());
        }

        return representation;
    }


    /**
     * Creates a stack of all N-grams (features) from a document.
     * @param document a string with all words from document
     * @param ngrams_depth the count of words in N-gram
     * @return a stack of all N-grams
     */
    private Stack<NGrams> cutDocumentToNGrams(String document, int ngrams_depth) {
        Stack<NGrams> ngrams = new Stack<NGrams>();

        //////////////////////////////////////////////////////////////////////////////////
        //A string of tokens with separators between sentences is created for a document//
        //////////////////////////////////////////////////////////////////////////////////
        String document_tokens = document.replaceAll("\\.", " . ");
        StringTokenizer split_list = new StringTokenizer(document_tokens);




        /////////////////////////////////////////////////////////////////
        //                              TREE                           //
        /////////////////////////////////////////////////////////////////
        //Root of the tree
        RootNode root_node = new RootNode();
        //Parent list to store the previous occurence of a node
        Stack<ChildNode> parent_list;
        //Variable needed to create the tree
        String word;


        //Creation of a suffix tree - scans all word in a document
        while(split_list.hasMoreTokens()) {
            //Clean the parent list
            parent_list = new Stack<ChildNode>();

            //Read first word
            word = split_list.nextToken();

            while((word != null) && !word.equals(".")) {
                //Is this word already in root node?
                ChildNode node = root_node.child_nodes.get(word);

                //If not, it is created
                if(node == null) {
                    node = new ChildNode(word, ChildNode.FIRST_OCCUR, ChildNode.ROOT_DEPTH);
                    root_node.child_nodes.put(word, node);
                }
                else {
                    node.count_occur = node.count_occur + 1;  //Increase of word occurence
                }

                //Add the found node into the parent list
                Stack<ChildNode> new_parent_list = new Stack<ChildNode>();
                new_parent_list.add(node);



                //Parent list is scanned and actual node is added to all predecessors
                //maximum depth of the tree is taken into consideration
                while(!parent_list.empty()) {
                    ChildNode parent_node = parent_list.pop();

                    //Detects, if a paren_node has a child with the same word
                    node = parent_node.child_nodes.get(word);

                    //If a node for a word does not exist, it is created
                    if(node == null) {
                        node = new ChildNode(word, ChildNode.FIRST_OCCUR, parent_node.depth + 1);
                        node.ngram = parent_node.ngram + " " + node.word;
                        parent_node.child_nodes.put(word, node);
                    }
                    else {
                        node.count_occur = node.count_occur + 1;  //Increase of word occurence
                    }

                    //Adds a reference to the next parent node, if the depth enables it
                    if(node.depth < ngrams_depth) {
                        new_parent_list.add(node);
                    }
                }



                //Setting of actual parent list
                parent_list = new_parent_list;

                //Reas the next word from a document
                try {
                    word = split_list.nextToken();
                }
                catch(NoSuchElementException ex) {
                    word = null;
                }
            }
        }


        //********************************************************************//
        //                       EXTRACTION OF NGRAM                          //
        //********************************************************************//
        Stack<ChildNode> extract_ngrams = new Stack<ChildNode>();
        extract_ngrams.addAll(root_node.child_nodes.values());


        //Scans the suffix tree and all N-grams are extracted
        while(!extract_ngrams.empty()) {
            ChildNode node = extract_ngrams.pop();

            //Adds an actual N-gram into the result
            if((node.depth == 1) || (node.count_occur >= min_ngram_occur)) {
                ngrams.add(new NGrams(node.ngram, node.count_occur));
            }

            //Adds all its childs into stack extract_ngrams to be in the result
            extract_ngrams.addAll(node.child_nodes.values());
        }


        return ngrams;
    }

    /**
     * Sets a parameter of minimum N-gram occurence
     * @param min_ngram_occur minimum N-gram occurence
     */

    public void setMinNGramOccur(int min_ngram_occur) {
        this.min_ngram_occur = min_ngram_occur;
    }






    /**
     * Creation of word representation (if the user didn't set a depth parameter....)
     * @param database database of all documents
     * @return created representation
     */
    public DocsRepresentDB buildRepresentation(DocumentsDatabase database) {
        BuilderWord build_word = new BuilderWord(model, preprocessing);
        return build_word.buildRepresentation(database);
    }


    @Override
    protected Void doInBackground() throws Exception {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
