/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.builder;

import java.util.Stack;
import java.util.StringTokenizer;
import javax.swing.SwingWorker;
import tp.model.Document;
import tp.model.DocRepresentTable;
import tp.model.DocumentsDatabase;
import tp.model.DocsRepresentDB;

/**
 * Class, which makes representation of text data by words.
 * @author Ondrej Sabatka
 */

public class BuilderWord extends SwingWorker<Void, Void> implements BuilderInterface {

    //Clenske promenne tridy
    private DocsRepresentDB.RepresentationModel model;
    private DocsRepresentDB.RepresentationFeature feature;
    private DocsRepresentDB.Preprocessing preprocessing;

    /**
     * Constructor of the class
     * @param model model used for representation (binary, TF or TF/IDF)
     * @param preprocessing preprocessing options selected by the user
     */
    public BuilderWord(DocsRepresentDB.RepresentationModel model, DocsRepresentDB.Preprocessing preprocessing) {
        this.model = model;
        this.preprocessing = preprocessing;
        this.feature = DocsRepresentDB.RepresentationFeature.WORD;
    }


    /**
     * Gets text documents from a database and creates their word representation.
     * @param database input database of text documents
     * @return database of all document representations
     */
    public DocsRepresentDB buildRepresentation(DocumentsDatabase database) {
        DocsRepresentDB representation = new DocsRepresentDB(model, feature, preprocessing);

        //Takes all documents and creates their representation
        for(int i = 0; i < database.getCountDocuments(); i++) {
            Document text_doc = database.getDocumentAt(i);

            //Document is cut into simple words
            Stack<String> features = cutDocumentToFeatures(text_doc.getContentDocument());

            //Table for document representations is created
            DocRepresentTable doc_table = new DocRepresentTable();

            //Individual features are inserted into the table of representations
            while(!features.empty()) {
                doc_table.includeFeature(features.pop(), model);
            }

            //Document class is stored into the table
            doc_table.setTopic(text_doc.getTopic());

            //Insertion of table with document representation into database of all representations
            representation.addDocument(doc_table);

            //Setting of a progress bar
            setProgress(i * 100 / database.getCountDocuments());
        }

        return representation;
    }


    /**
     * Creates a stack of all words (features) contained in a document.
     * @param document a string with all words from document
     * @return a stack of all features
     */
    private Stack<String> cutDocumentToFeatures(String document) {
        Stack<String> features = new Stack<String>();

        //Removal of dots "." from a document - not necessary for word representation
        document = document.replaceAll("\\.", "");

        StringTokenizer st = new StringTokenizer(document);

        //Vector is filled with all words...
        while(st.hasMoreTokens()) {
            features.push(st.nextToken());
        }

        return features;
    }




    /**
     * Mthod creates a representation, if the depth is equal to "1" - because this class creates word
     * representation (higher depth is possible only for N-gram representation).
     * @param database database with all documents
     * @param depth depth (length of a feature) - "1" for word representation
     * @return created representation
     */
    public DocsRepresentDB buildRepresentation(DocumentsDatabase database, int depth) {
        if(depth == 1) {  //User must set the depth to '1'
            return buildRepresentation(database);
        }
        else {  //if the depth is not 1, it is an error
            return null;
        }
    }

    @Override
    protected Void doInBackground() throws Exception {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void setMinNGramOccur(int min_ngram_occur) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
}
