/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.controllers;

import java.awt.Cursor;
import java.io.File;
import javax.swing.SwingWorker;
import tp.model.Document;
import tp.model.DocumentsDatabase;
import tp.model.DocsRepresentDB;
import tp.preprocess.PreprocessDocument;
import tp.builder.BuilderInterface;
import tp.builder.BuilderNGram;
import tp.builder.BuilderStatistics;
import tp.builder.BuilderVector;
import tp.builder.BuilderWord;
import tp.view.ProgressView;

/**
 * A controller to process events on a screen, where the setting of pre-processing and
 * representation of documents.
 * @author Ondrej Sabatka
 */

public class SettingsControl extends ActionControl {

    //Varaibles
    private DocumentsDatabase database;
    private DocsRepresentDB representation_database;

    //Parameters of pre-processing
    private boolean use_stop_list;
    private boolean use_own_stop_list;
    private boolean use_stemming;

    private File own_stop_list_path;

    //Text document representation settings
    public enum TypeOfRepresentationModels { BINARY, TF, TFIDF }
    private static String[] representation_model_names =
                            {"Binary representation", "TF Model", "TF-IDF Model"};
    public enum TypeOfRepresentationFeatures { WORDS, NGRAMS }
    private static String[] representation_feature_names =
                            {"Words", "N-Grams"};

    //Varaibles used to stroe settings
    private int representation_model;
    private int select_representation_feature;
    
    //N-grams settings
    private int ngram_depth;
    private int min_ngram_occur;

    public static final int WORD_DEPTH = 1;
    public static final int NGRAM_MIN_DEPTH = 2;
    public static final int NGRAM_MAX_DEPTH = 5;

    public static final int NGRAM_MIN_OCCUR = 1;
    public static final int NGRAM_MAX_OCCUR = 4;
    public static final int NGRAM_DEFAULT_OCCUR = 2;


    /**
     * Constructor of the class
     */
    public SettingsControl() {
        //Inicializace parametru predzpracovani
        this.use_stop_list = true;
        this.use_own_stop_list = false;
        this.use_stemming = true;
        this.select_representation_feature = TypeOfRepresentationFeatures.WORDS.ordinal();
        this.representation_model = TypeOfRepresentationModels.BINARY.ordinal();

        this.ngram_depth = NGRAM_MIN_DEPTH;
        this.min_ngram_occur = NGRAM_DEFAULT_OCCUR;
    }


    /**
     * Initialization - setting of reference to database of documents
     * @param database database of documents
     */
    public void init(DocumentsDatabase database) {
        this.database = database;
    }



    /**
     * A class, which ensures execution of pre-processing in the background.
     */
    class Task extends SwingWorker<Void, Void> {

        /**
         * A method which initializes and executes pre-processing in the background.
         */
        @Override
        public Void doInBackground() {
            //Cursor is set to wait and progress bar is initialized
            MainControl.getInstance().getMainView().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            ProgressView progress = MainControl.getInstance().getMainView().getProgressBar();

            //If user chose stemming or stoplist, it is executed here...
            if(use_stemming || use_stop_list) {
                progress.start("Document pre-processing");
                this.addPropertyChangeListener(progress);

                //Preprocessing obejct
                PreprocessDocument preprocess = null;

                //Reading of user-defined stoplist
                if(use_own_stop_list) {
                     preprocess = new PreprocessDocument(own_stop_list_path);
                }
                else {
                    preprocess = new PreprocessDocument();
                }

                Document doc = null;
                int count_documents = database.getCountDocuments();

                //Pre-processing execution for each document
                for(int i = 0; i < count_documents; i++) {
                    doc = database.getDocumentAt(i);
                    doc = preprocess.preprocessDocument(doc, use_stop_list, use_stemming);
                    database.setDocoumentTo(i, doc);

                    //nformation for progress bar
                    setProgress(i * 100 / count_documents);
                }
            }


            /******************************************************************
             *                MODEL AND REPRESENTATION CREATION               *
             ******************************************************************/
            progress.start("Creating of document representation");

            //Pre-processing parameters setting
            DocsRepresentDB.RepresentationModel model = null;
            DocsRepresentDB.Preprocessing preprocessing = null;

            if(use_stemming && use_stop_list) {
                preprocessing = DocsRepresentDB.Preprocessing.STOP_LIST_STEMMING;
            }
            else if(use_stemming) {
                preprocessing = DocsRepresentDB.Preprocessing.STEMMING;
            }
            else if(use_stop_list) {
                preprocessing = DocsRepresentDB.Preprocessing.STOP_LIST;
            }
            else {
                preprocessing = DocsRepresentDB.Preprocessing.NONE;
            }

            //Model parameters setting
            if(representation_model == TypeOfRepresentationModels.BINARY.ordinal()) {
                model = DocsRepresentDB.RepresentationModel.BINARY;
            }
            else if(representation_model == TypeOfRepresentationModels.TF.ordinal()) {
                model = DocsRepresentDB.RepresentationModel.TF;
            }
            else {
                model = DocsRepresentDB.RepresentationModel.TFIDF;
            }


            //Execution of the process
            BuilderInterface builder;
            int depth;

            if(select_representation_feature == TypeOfRepresentationFeatures.NGRAMS.ordinal()) {
                builder = new BuilderNGram(model, preprocessing);
                builder.setMinNGramOccur(min_ngram_occur);
                depth = ngram_depth;
            }
            else {  //WORD
                builder = new BuilderWord(model, preprocessing);
                depth = WORD_DEPTH;
            }

            //Setting of listener for progress bar
            builder.addPropertyChangeListener(progress);

            //Execution of the process of creating representation
            representation_database = builder.buildRepresentation(database, depth);


            /******************************************************************
             *                 COUNTING OF TF OR TF-IDF                       *
             ******************************************************************/
            if(representation_model == TypeOfRepresentationModels.TF.ordinal()) {
                progress.start("Counting TF value");

                BuilderVector calculate_tfidf = new BuilderVector();
                representation_database = calculate_tfidf.calculateTF(representation_database);
            }
            else if(representation_model == TypeOfRepresentationModels.TFIDF.ordinal()) {
                progress.start("Counting TF-IDF value");
                
                BuilderVector calculate_tfidf = new BuilderVector();
                representation_database = calculate_tfidf.calculateTFIDF(representation_database);
            }


            /******************************************************************
             *                        COUNTING OF STATISTICS                  *
             ******************************************************************/
            progress.start("Counting statistics");

            BuilderStatistics calculate_statistics = new BuilderStatistics();
            representation_database = calculate_statistics.calculateStatistics(representation_database);

            
            return null;
        }


        /**
         * This called after finishing of data pre-processing. It changes setting of
         * progress bar and cursor.
         */
        @Override
        public void done() {
            MainControl.getInstance().getMainView().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            MainControl.getInstance().getMainView().getProgressBar().stop();

            //Switch to the tab with results
            if(representation_database != null) {
                MainControl.getInstance().gotoResults(representation_database);
            }
            else {
                MainControl.getInstance().getMainView().
                        showErrorDialog("Creating of document representation failed!");
            }
        }
    }




    /*
     ********************************************************************
     *                      GUI EVENTS HANDLING                         *
     ********************************************************************
     */


    /**
     * After click of "Next", a pre-processing task is executed.
     */
    public void nextBtnClick() {
        Task task = new Task();
        task.execute();
    }


    /**
     * The method is executed after change of a check-box for stop-list. Corresponding
     * variable is modified.
     */
    public void useStopListChange() {
        this.use_stop_list = !this.use_stop_list;

        if(this.use_stop_list) {
            MainControl.getInstance().getMainView().getTabView().getTabSettings().isUseStopList();
        }
        else {
            use_own_stop_list = false;
            MainControl.getInstance().getMainView().getTabView().getTabSettings().isNotUseStopList();
        }
    }


    /**
     * The method is executed after change of a check-box for stemming. Corresponding
     * variable is modified.
     */
    public void useStemmingChange() {
        this.use_stemming = !this.use_stemming;
    }


    /**
     * The method is executed after change of a chosen feature for document representation.
     */
    public void changeFeature(String type) {
        this.select_representation_feature = Integer.parseInt(type);
        MainControl.getInstance().getMainView().getTabView().getTabSettings().changeFeature();
    }


    /**
     * The method is executed after change of a chosen model for document representation.
     */
    public void changeModel(String type) {
        this.representation_model = Integer.parseInt(type);
    }


    /**
     * The method is executed after change of N-gram size.
     * It stores actual maximum size of N-gram.
     */
    public void setNgramDepth() {
        this.ngram_depth =
                MainControl.getInstance().getMainView().getTabView().getTabSettings().getMaxNgramDepth();
    }


    /**
     * The method is executed after change of N-gram size.
     * It stores actual minimum size of N-gram.
     */
    public void setNgramOccur() {
        this.min_ngram_occur =
                MainControl.getInstance().getMainView().getTabView().getTabSettings().getMinNgramOccur();
    }

    

    /*
     ********************************************************************
     *                          GETTERs A SETTERs                       *
     ********************************************************************
     */

    /**
     * The method sets a variable with information, if stemming will be used.
     * @param use_stemming true, if stemming is used, false otherwise
     */
    public void setStemming(boolean use_stemming) {
        this.use_stemming = use_stemming;
    }


    /**
     * The method gets information, if stemming will be used.
     * @return true, if stemming is used, false otherwise
     */
    public boolean useStemming() {
        return use_stemming;
    }

    /**
     * The method gets information, if user-defined stoplist will be used.
     * @return true, if user-defined stoplist is used, false otherwise
     */
    public boolean useOwnStopList() {
        return use_own_stop_list;
    }

    /**
     * The method is used to set path to user-defined stoplist.
     */
    public void ownStopListChange() {
        use_own_stop_list = !use_own_stop_list;

        //Open dialog opens, if user wants to use his own stoplist
        if(use_own_stop_list) {
            this.own_stop_list_path =
                    MainControl.getInstance().getMainView().getTabView().getTabSettings().getStopListPath();

            if(own_stop_list_path != null) {

                MainControl.getInstance().getMainView().getTabView().getTabSettings().setOwnStopListName(
                        own_stop_list_path.getName());
                
            }
            else {
                use_own_stop_list = false;
            }
        }
        else {
            MainControl.getInstance().getMainView().getTabView().getTabSettings().setOwnStopListClear();
        }
    }


    /**
     * The method sets a variable with information, if stoplist will be used.
     * @param use_stop_list true, if stoplist is used, false otherwise
     */
    public void setStopList(boolean use_stop_list) {
        this.use_stop_list = use_stop_list;
    }


    /**
     * The method gets information, if stoplist will be used.
     * @return true, if stoplist is used, false otherwise
     */
    public boolean useStopList() {
        return use_stop_list;
    }


    /**
     * Metoda vrati pole retezcu s nazvy rysu, ktere lze pouzit pro reprezentaci
     * dokumentu
     * @return pole retezcu s nazvy rysu
     */
    public static String[] getRepresentationFeatureNames() {
        return representation_feature_names;
    }


    /**
     * The method returs an array with names of models, which can be used
     * for document representation.
     * @return array of strings with model names
     */
    public static String[] getRepresentationModelNames() {
        return representation_model_names;
    }


    /**
     * The method gets information about feature, which is used to represent documents.
     * @return representation feature
     */
    public int getRepresentationFeature() {
        return select_representation_feature;
    }


    /**
     * The method gets information about model, which is used to represent documents.
     * @return representation model
     */
    public int getRepresentationModel() {
        return representation_model;
    }

}
