/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.model;

import java.util.ArrayList;
import java.util.Hashtable;

/**
 * Represents a table of one document representation.
 * @author Ondrej Sabatka
 */

public class DocRepresentTable {

    //Variables of the class
    private Hashtable<String, FeatureInterface> features;
    private String topic;

    private static final int FIRST_PRESENCE = 1;


    /**
     * Constructor of the class.
     */
    public DocRepresentTable() {
        //Table of features initialization
        this.features = new Hashtable<String, FeatureInterface>();
    }


    /**
     * Method adds a new feature to the representation, if it does not exist. 
     * If it exists, its occurence is increased.
     * @param f a feature
     * @param model model of representation (binary/tf/tf-idf)
     */
    public void includeFeature(String f, DocsRepresentDB.RepresentationModel model) {
        //Searching for an object with feature f
        FeatureInterface feature = this.features.get(f);

        //If it is not contained in this document
        if(feature == null) {
            if(model == DocsRepresentDB.RepresentationModel.BINARY) {
                this.features.put(f, new FeatureBinary(f));  //New feature is stored
            }
            else {
                this.features.put(f, new FeatureFrequency(f, FIRST_PRESENCE));  
            }
        }
        //It already exists here, only occurence is increased
        else if(model != DocsRepresentDB.RepresentationModel.BINARY) {
            feature.setRate(feature.getRate() + 1);  //Increment occurence number
        }
    }


    /**
     * Method inserts directly features and their frequencies
     * @param feature a feature (word)
     * @param frequency frequency of a feature
     */
    public void addFeature(String feature, Integer frequency) {
        //Stores a new feature to a new position
        this.features.put(feature, new FeatureFrequency(feature, frequency));
    }


    /**
     * Method inserts directly features to a binary representation
     * @param feature a new feature
     */
    public void addFeature(String feature) {
         //Stores a new feature to a new position
        this.features.put(feature, new FeatureBinary(feature));
    }


    /**
     * Method returns a term frequency a feature, wich is given as a parameter
     * @param feature a given feature
     * @return term frequency of a feature
     */
    public double getFeatureTF(String f) {
        FeatureFrequency fe = (FeatureFrequency) this.features.get(f);

        if(fe == null) {
            return -1.0;
        }

        return fe.getTF();
    }

    /**
     * Method returns a TFIDF weight a feature, which is given as a parameter
     * @param feature a given feature
     * @return TFIDF weight of a feature
     */
    public double getFeatureTFidf(String f) {
        FeatureFrequency fe = (FeatureFrequency) this.features.get(f);

        if(fe == null) {
            return -1.0;
        }

        return fe.getTFidf();
    }


    /**
     * Method returns occurence of a feature (for binary model)
     * @param feature a given feature
     * @return (binary) occurence of a feature
     */
    public int getFeatureBinary(String f) {
        FeatureBinary fe = (FeatureBinary) this.features.get(f);
        
        if(fe == null) {
            return -1;
        }

        return fe.getRate();
    }


    /**
     * Method returs the whole representation of a document with all features.
     * @return array of features and their frequencies
     */
    public ArrayList<FeatureInterface> getArrayFeaturesFrequency() {
        ArrayList<FeatureInterface> array = new ArrayList<FeatureInterface>();
        array.addAll(this.features.values());
        return array;
    }
    
    /**
     * Returns a class of represented document.
     * @return a class in a form of string
     */
    public String getTopic() {
        return topic;
    }

   /**
     * Sets a class of represented document.
     * @param a class in a form of string
     */
    public void setTopic(String topic) {
        this.topic = topic;
    }

}
