/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.preprocess;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.StringTokenizer;
import tp.controllers.MainControl;
import tp.model.Document;

/**
 * A class for pre-processing of documents. This includes stemming and stop words removal.
 * @author Ondrej Sabatka
 */

public class PreprocessDocument {

    private Stemmer stemmer;
    private ArrayList<String> stop_words;

    //Konstruktory
    public PreprocessDocument() {
        this.stemmer = new Stemmer();
        this.stop_words = new ArrayList<String>();

        loadCommonStopList();
    }

    public PreprocessDocument(File stop_list) {
        this.stemmer = new Stemmer();
        this.stop_words = new ArrayList<String>();

        loadStopListFromFile(stop_list);
    }


    /**
     * The method makes preprocessing of one dcument.
     * @param doc a document to be pre-processed
     * @param use_stop_list a boolean value, which indicates, if stoplist will be used
     * @param use_stemming a boolean value, which indicates, if stemming will be used
     * @return
     */
    public Document preprocessDocument(Document doc, boolean use_stop_list, boolean use_stemming) {
        Document preproc_d = null;
        StringTokenizer st = new StringTokenizer(doc.getContentDocument());

        String preprocess_content = new String("");
        String word = null;
        boolean remove_dot;

        //Reading of each word in a document
        while(st.hasMoreElements()) {
            word = st.nextToken();

            //If "." is at the end of word, it is removed ...
            //and then it is returned - char "." is important for N-grams
            remove_dot = false;
            //Is "." at the end?
            if(word.charAt(word.length() - 1) == '.') {
                //Remove it...
                word = word.substring(0 , word.length() - 1);
                remove_dot = true;
            }


            //STOP LIST
            if(use_stop_list) {
                if(isStopWord(word)) {
                    word = "";
                }
            }


            //STEMMING
            if(use_stemming) {
                word = this.stemmer.stem(word);
            }


            //Saving pre-processed word
            preprocess_content = preprocess_content + " " + word;
            preprocess_content = preprocess_content.trim();

            if(remove_dot) {
                preprocess_content = preprocess_content + ".";
            }

        }

        //Store a pre-processed document into a new document
        //and return it
        preproc_d = new Document(preprocess_content, doc.getTopic());
        return preproc_d;
    }


    /**
     * Tests, if a word is in the stoplist.
     * @param str a string with a word
     * @return boolean value, true, if a word is in the stoplilst, false otherwise
     */
    private boolean isStopWord(String str) {
        if (Collections.binarySearch(stop_words, str) >= 0) {
            return true;
        }
        else {
            return false;
        }
    }

    
    /**
     * Reads a stoplist from sources (tp.recources.stop-list) and an array stop_list
     * is filled with its words.
     */
    private void loadCommonStopList() {
        BufferedReader fin = null;
        String line = null;
        
        try {
            //Input stream for reading
            fin = MainControl.getInstance().getResourceModel().getStopListPath();

            //File content is stored to content_file string
            while ((line = fin.readLine()) != null) {
                //Reads individual words in a row and cleans it from non-content chars
                StringTokenizer st = new StringTokenizer(line, ";");

                while (st.hasMoreTokens()) {
                    this.stop_words.add(st.nextToken());
                }
            }
        } catch (FileNotFoundException e) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("Error during reading of stoplist!");
        }
        catch (IOException e) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("Error during reading of stoplist!");
        }
    }

    /**
     * Reads a stoplist from file chosen by the user.
     * @param stop_list a file containing a stoplist
     */
    private void loadStopListFromFile(File stop_list) {
        BufferedReader fin = null;
        String line = null;

        try {
            //Input stream for reading
            fin = new BufferedReader(new FileReader(stop_list));

            //File content is stored to content_file string
            while ((line = fin.readLine()) != null) {
                //Reads individual words in a row and cleans it from non-content chars
                StringTokenizer st = new StringTokenizer(line, ";");

                while (st.hasMoreTokens()) {
                    this.stop_words.add(st.nextToken());
                }
            }
        } catch (FileNotFoundException e) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("Error during reading of stoplist!");
        }
        catch (IOException e) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("Error during reading of stoplist!");
        }
    }
}
