/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.store;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Formatter;
import javax.swing.SwingWorker;
import tp.controllers.MainControl;
import tp.model.DocRepresentTable;
import tp.model.DocsRepresentDB;
import tp.model.DocsRepresentDB.FeatureDF;
import tp.model.FeatureBinary;

/**
 * Class for saving pre-processed documents in CSV files.
 * @author Ondrej Sabatka
 */

public class StoreCSV extends SwingWorker<Void, Void> implements StoreInterface {

    //Varaibles
    String separator;

    /**
     * Constructor of the class
     * @param separator separator of values in a CSV file
     */
    public StoreCSV(String separator) {
        this.separator = separator;
    }

    /**
     * Writes the whole database of document representations into a CSV file.
     * @param file a file, into which the representations will be written
     * @param features_array array of all features (1st row in file)
     * @param representation_database database of all document representations
     * @param write_topic boolean value, if the categories of documents are stored
     */
    public void writeDocRepresentation(File file, ArrayList<FeatureDF> features_array, DocsRepresentDB representation_database, boolean write_topic) {
        try {
            //Opening of the file
            BufferedWriter fout = new BufferedWriter(new FileWriter(file)); //Bufrovany zapis

            //Heading of the file
            //ID ; class ;  all features separated by ;
            if(write_topic) {
                fout.write("ID" + this.separator + "topic");
            }
            else {
                fout.write("ID");
            }

            //writing of feature list
            for(int i = 0; i < features_array.size(); i++) {
                DocsRepresentDB.FeatureDF f = features_array.get(i);

                fout.write(this.separator + f.feature);
            }


            //End of first row
            fout.write("\n");

            //Writing all documents (weights af all their features)
            for(int i = 0; i < representation_database.getCountDocuments(); i++) {
                DocRepresentTable doc = representation_database.getDocumentAt(i);

                //ID + class (optionally)
                if(write_topic) {
                    //If class does not exist
                    if(doc.getTopic().equals("")) {
                        continue;
                    }

                    fout.write("D" + i + this.separator + doc.getTopic());
                }
                else {
                    fout.write("D" + i);
                }

                //Writing of all weights; according to chosen model
                if(representation_database.getModel() == DocsRepresentDB.RepresentationModel.BINARY) {
                    writeBinaryFeatureValues(fout, features_array, doc);
                }
                else if(representation_database.getModel() == DocsRepresentDB.RepresentationModel.TF) {
                    writeTFFeatureValues(fout, features_array, doc);
                }
                else {
                    writeTFidfFeatureValues(fout, features_array, doc);
                }
                

                
                fout.write("\n");
                setProgress(i * 100 / representation_database.getCountDocuments());
            }


            //Close the file
            fout.close();

        } catch (IOException ex) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("Saving into file was not successful!");
        }
    }

    /**
     * Writes one binary weight vector into a file.
     * @param fout reference to an opened file
     * @param features_array array of all features
     * @param doc binary vector with representation of one document
     */
    private void writeBinaryFeatureValues(BufferedWriter fout, ArrayList<FeatureDF> features_array, DocRepresentTable doc) throws IOException {

        for(int fi = 0; fi < features_array.size(); fi++) {
            DocsRepresentDB.FeatureDF feature = features_array.get(fi);

            int value = doc.getFeatureBinary(feature.feature);

            if(value < 0) {
                fout.write(this.separator + "0");
            }
            else {
                fout.write(this.separator + FeatureBinary.PRESENCE);
            }
        }
    }

    /**
     * Writes one TF (term frequency) weight vector into a file.
     * @param fout reference to an opened file
     * @param features_array array of all features
     * @param doc TF vector with representation of one document
     */
    private void writeTFFeatureValues(BufferedWriter fout, ArrayList<FeatureDF> features_array, DocRepresentTable doc) throws IOException {

        for(int fi = 0; fi < features_array.size(); fi++) {
            DocsRepresentDB.FeatureDF feature = features_array.get(fi);

            double value = doc.getFeatureTF(feature.feature);

            if(value < 0.0) {
                fout.write(this.separator + "0");
            }
            else {
                Formatter formatter = new Formatter();
                String svalue = formatter.format("%.5f", value).toString().replaceAll(",", ".");
                fout.write(this.separator + svalue);
            }
        }
    }

    /**
     * Writes one TF-IDF (term frequency/inverse document frequency) weight vector into a file.
     * @param fout reference to an opened file
     * @param features_array array of all features
     * @param doc TF-IDF vector with representation of one document
     */
    private void writeTFidfFeatureValues(BufferedWriter fout, ArrayList<FeatureDF> features_array, DocRepresentTable doc) throws IOException {

        for(int fi = 0; fi < features_array.size(); fi++) {
            DocsRepresentDB.FeatureDF feature = features_array.get(fi);

            double value = doc.getFeatureTFidf(feature.feature);

            if(value < 0.0) {
                fout.write(this.separator + "0");
            }
            else {
                Formatter formatter = new Formatter();
                String svalue = formatter.format("%.5f", value).toString().replaceAll(",", ".");
                fout.write(this.separator + svalue);
            }
        }
    }



    @Override
    protected Void doInBackground() throws Exception {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
